/*
**  $VER: mycodepic.c 0.003 (13 Nov 1998)
**
**   Ian Stedman 1998
**
**  PROGRAMNAME:			mycodepic.c
**
**  FUNCTION:				Code generator for the PIC_CC Compiler
**
**  $HISTORY:
**
** 2/16/95					 generates less inline code
** 2/15/95					 minor glitch in gsub
** 8/8/97					 modified to work with MPASM
** 13/11/97 	  Still re-writing to use MPASM syntax. Now using the trace
**   code. A few instructions to implement plus stack code to be done
** 4th October 1998		changing the code generator to output code that is
**   compatible with PICAsm as this is the assembler that I use! It will
**   allow the compiler to call the assembler and generate a INHX8M object.
**
**  Original PIC C Version by  John Favata 1995
**   13 Nov 1998 : 000.003 : work in progress.
*/


#include <stdio.h>

#include "defs.h"

#include "data.h"

#include "version.h"

#define INCDIR "PROGDIR:"


#define ERRMSG "FATAL ERROR in code generation\n"
#define WARNING "; WARNING: Signed chars are not implemented\n"
#define CHAR_WARNING "; WARNING: only char ptrs are allowed\n"

#define STORE_START 0x0C

// Used to setup start of data storage.


#define STACK 0x24	  // Start of the Stack in Memory
#define MAXSTACK 8	  // Max stack depth

/* Define ASNM and LDNM to the names of the assembler and linker respectively */
/* ASMNAME = PICASM		*/

/*
 * 				  Some predefinitions:
 *
 * 				  INTSIZE is the size of an integer in the target machine
 * 				  BYTEOFF is the offset of an byte within an integer on the
 * 				  target machine. (ie: 8080,pdp11 = 0, 6809 = 1, 360 = 3)
 * 				  This compiler assumes that an integer is the SAME length as
 * 				  a pointer - in fact, the compiler uses INTSIZE for both.
 */

#define INTSIZE 1
#define BYTEOFF 0

FILE *Trace;

int TRACE = 1;

int avail_store_addr = STORE_START;
int stack_addr 		= STACK; // set intial stack value

int M_lib=FALSE;								  /* see if we have to load MULT library */
int S_lib=FALSE;								  /* see if we need shift library */
int G_lib=FALSE;								  /* see if we need comp library */

char stacky[8][7] = {"STACK0", "STACK1", "STACK2", "STACK3", "STACK4", "STACK5", "STACK6", "STACK7" };




header ()
{
						if (TRACE==1)
								  {
											 fprintf(output,"\t;header()\n");
										  }
						fprintf(output,";\tSmall C for the PIC16C/F84;\n;\tCompiled on %s \n", COMPILEDATE);
						fprintf(output,";\tVersion %s\n",VERSION);
						outstr (";\tBy Ian Stedman. ICStedman@techie.com");
						if (TRACE==1)
										  {
													  printf ("MYCODEPIC.C compiled on %s at %s", __DATE__, __TIME__);
										  }
						nl ();
						outstr ("\t#include \"16c84.h\" ");
						nl ();
}

nl ()
{
						outbyte (EOL);
}


galign(int t)
{
						if (TRACE)
													  fprintf(output,";galign(%d)\n",t);
						return(t);
}

/*
 * 				  return size of an integer
 */
intsize() {
						return(INTSIZE);
}

/*
 * 				  return offset of ls byte within word
 * 				  (ie: 8080 & pdp11 is 0, 6809 is 1, 360 is 3)
 */

byteoff() {
						return(BYTEOFF);
}

/*
 * 				  Output internal generated label prefix
 */
olprfix() {
						outbyte(' ');
}

/*
 * 				  Output a label definition terminator
 */
col ()
{
						outbyte (' ');
}

/*
 * 				  begin a comment line for the assembler
 *
 */
comment ()
{
						outbyte (';');
}

/*
 * 				  Emit user label prefix
 */
prefix ()
{
														outbyte(' ');
}


/*
 * 				  print any assembler stuff needed after all code
 *
 */
trailer ()
{
						if (TRACE)
								 fprintf(output,"\t\n;trailer()\n");

						ol ("end");
						nl();
}

/*
 * 				  function prologue
 */
prologue ()
{
						if (TRACE)
													  fprintf(output,"\t;prologue()\n");
						ol("***************************");
						ol("; Begin Subroutine \n");
						ol("\n");
}

/*
 * 				  text (code) segment
 */
gtext ()
{
						if (TRACE)
													  fprintf(output,"\t;gtext()\n");
						ol ("\n; *****CODE Section*********");
						ol (";  The start of your code! \n");
						ol ("org 0x04\n");
}

/*
 * 				  data segment
 */
gdata ()
{
						if (TRACE) fprintf(output,"\t;gdata()\n");
																if (M_lib==TRUE) {
																					  outstr("\n;*****need mult/div standard library*****\n");
																					  ol("\t#include \"mlibpic.h\" \n");
																}

																if (S_lib==TRUE) {
																					  outstr("\n;*****need shift/sub standard library****\n");
																					  ol("\t#include \"slibpic.h\" \n");
																}

																if (G_lib==TRUE) {
																					  outstr("\n;****need compare standard library****\n");
																					  ol("\t#include \"glibpic.h\" \n");
																}

						ol ("\n; *******data segment dseg*************");
						ol (" ; Data storage starts at 0x0C for the 16C84");
						ol ("  org 0x0C\n");
}

/*
 *  Output the variable symbol at scptr as an extrn or a public
 *  Re-enabled the last 2 if statements 4/10/98
 */
ppubext(char *scptr)
{
						if (TRACE)
								 fprintf(output,"\t;ppubext(%s)\n",scptr);

						//if (cptr[STORAGE] == STATIC) return;
						//ot (scptr[STORAGE] == EXTERN ? "extrn\t" : "public\t");
						//prefix ();

						//outstr (scptr);
						//nl();

}

/*
 * Output the function symbol at scptr as an extrn or a public
 * RE-enabled on 3/4/98.
 */
fpubext(char *scptr)
{
						if (TRACE)
													  fprintf(output,"\t;ppubext(%s)\n",scptr);

						if (scptr[STORAGE] == STATIC) return;
						ot (scptr[OFFSET] == FUNCTION ? "public\t" : "extrn\t");
						prefix ();
						outstr (scptr);
						nl ();

}

/*
 *  Output a decimal number to the assembler file
 */
onum(int num)
{
						outdec(num);
}


/*
 * 				  fetch a static memory cell into the working register
 */
getmem (char *sym)
{
						if (TRACE){
											 fprintf(output,"\t;getmem(%s)\n",sym);
											 }

						if ((sym[IDENT] != POINTER) &
											(sym[TYPE] == CCHAR)) {
											//ot ("movlw ");
											// next line was original code
											//outstr (sym + NAME);

													  fprintf(output,"\tmovlw %s\n",sym);
													  nl ();
													  /* gcall ("?sxt"); */
											 }
								  else {
													  outstr("char pointer warning");
													  fprintf(output,"\tmovlw %d",&sym);
													  //outstr (sym + NAME);
													  nl ();
											 }
}
/*
 * 				  fetch the popress of the specified symbol into the primary register
 *
 */
getloc (char *sym)
{
						if (TRACE)
							  {
							  fprintf(output,"\t;getloc(%s)\n",sym);
							  fprintf(output,"\t; call made to immed from mycodepic.c line %d",__LINE__);
							  }
						immed(*sym);

						if (sym[STORAGE] == LSTATIC) {
													  printlabel(glint(sym));
													  nl();
						} else {
													  outdec (glint(sym) - stkp ); /**/
													  nl ();
													  /* outstr (";*****dad sp****\n"); */
													  ol ("popwf primary, stackptr");
						}
}

/*
 * 				  store the primary register into the specified static memory cell
 *
 */
putmem (char *sym)
{
						if (TRACE) {
													  fprintf(output,"\t;putmem(%s)\n",sym);
													  }

						if ((sym[IDENT] != POINTER) &
														(sym[TYPE] == CCHAR)) {
													  fprintf(output,"\tmovwf %s",sym);
													 // outstr (sym + NAME);
													 // col();
						} else {
													  outstr(CHAR_WARNING);
													 // ot ("movwf ");
													 // outstr (sym + NAME);
													 // col();
													  fprintf(output,"\tmovwf %s",sym);
						}
						nl ();
}

/*
 * 				  store the specified object type in the primary register
 * 				  at the address on the top of the stack
 *
 */
putstk (char typeobj)
{
						if (TRACE)
							 fprintf(output,"\t;putstk(%d)\n",typeobj);

						gpop ();
						if (typeobj == CCHAR) {
																											 gcall("putstk");
/****
													  ol( "movf fr4, secondary\n");
													  ol( "movf fr0, primary\n");
*****/
						}
						else {
																											 outstr(CHAR_WARNING);
																											 gcall("putstk");
/*****
														ol( "movf fr4, secondary\n");
														ol( "movf fr0, primary\n");
*****/
						}
}

/*
 * 				  fetch the specified object type indirect through the primary
 * 				  register into the primary register
 *
 */
indirect (char typeobj)
{
						if (TRACE){
													  fprintf(output,"\t;indirect(%d)\n",typeobj);
													  }

						if (typeobj == CCHAR) {
																/* 		ol("addf primary, stackptr");  */
																													 gcall("indr");
/****
																ol("movf fr4, primary");
																ol("movf primary, fr0");
****/
						}
						else {
																outstr(CHAR_WARNING);
																													 gcall("indr");
/***
																ol("movf fr4, primary");
																ol("movf primary, fr0");
****/
						}
}

/*
 * 				  swap the primary and secondary registers
 * 				  Removed/commented out the gcall.
 * 				  Changed on 13th November 1998.
 */
swap ()
{
						if (TRACE){
													  fprintf(output,"\t;swap\n");
													  }
						/* 							gcall("swap"); 		 */
						ol ("movwf temp");
						ol ("movwf second");
						ol ("movf temp,W");

}

/*
 * 				  print partial instruction to get an immediate value into
 * 				  the primary register
 * 				  IN PIC Assembler movlw does this! So it will be used.
 * 				  Check how to make a pointer work here! e.g. val.
 */

immed (int *val)
{
						if (TRACE)
								fprintf(output,"\t;immed()\n");
						fprintf(output,"\tmovlw %d\n",val);
}

immed_nopound (char *val)
{
						if (TRACE)
													  fprintf(output,"\t;immed()\n");
						fprintf(output,"\tmovlw %s",val);
}


/*
 * 				  push the primary register onto the stack
 * 				  Is this necessary? PIC Handles stack upto 8 levels.
 */
gpush ()
{
						if (TRACE)
						fprintf(output,"\t;gpush\n");
/*****
						ol ("dec stackptr");
						ol ("movf fr4, stackptr");
						ol ("movf fr0, primary");
******/
						gcall("push");
						stkp = stkp - INTSIZE;
}

/*
 * 				  pop the top of the stack into the secondary register
 * 				  Again commented out gcall and used code here 3/4/98.
 */
gpop ()
{
						if (TRACE)
													  fprintf(output,"\t;gpop\n");
						fprintf(output,"movf second",);

						/*ol ("movf secondary, fr0");
						ol ("inc stackptr");
						gcall("pop");	*/
						stkp = stkp + INTSIZE;
}



/*
 * 				  swap the primary register and the top of the stack
 *
 */
swapstk ()
{
						if (TRACE)
								 fprintf(output,"\t;swapstk\n");
																gcall("swaps");
/*****
						ol ("movf fr4, stackptr");
						ol ("movf temp, fr0");
						ol ("movf fr0, primary");
						ol ("movf primary, temp");
*****/
}

/*
 * 				  call the specified subroutine name
 *
 */
gcall (char *sname)
{
						if (TRACE)
													  fprintf(output,"\t;gcall\n",sname);
						ot ("call ");
						outstr (sname);
						col();
						nl ();
}

/*
 * 				  return from subroutine
 *
 */
gret ()
{
						if (TRACE)
													  fprintf(output,"\t;gret\n");
						ol ("return");
}

/*
 * 				  perform subroutine call to value on top of stack
 *
 */
callstk ()
{
						if (TRACE)
													  fprintf(output,"\t;callstk\n");
outstr( ERRMSG );
						return;
/******
						immed ();
						outstr ("$+5");
						nl ();
						swapstk ();
						ol ("pchl");
						stkp = stkp + INTSIZE;
******/
}

/*
 * 				  jump to specified internal label number
 *
 */
jump (int label)
{
						if (TRACE)
													  fprintf(output,"\t;jump(%d)\n",label);
						ot ("goto ");
						printlabel (label);
						col();
						nl ();
}

/*
 * 				  test the primary register and jump if false to label
 * 				  Ah conditional test. Need bit test operations.
 */
testjump (int label,
								 int ft)
{
						if (TRACE)
													  fprintf(output,"\t;testjump(%d,%d)\n",label,ft);
						ol ("movlw, primary");
						/* Get data into W */

						if (ft){
													  ot ("btfsc 3,2");
													  ot ("goto ");
													 }
						else
													 {
													  ot ("btfss 3,2");
													  ot ("goto ");
													 }
						/* BZ is a Microchip mnemonic for Branch on Zero. Equivalent
						to BTFSC 3,1  GOTO k */

						printlabel (label);
						col();
						nl ();
}

/*
 * 				  print pseudo-op 		 to define a byte
 * 				  Will start to fill the PICs EEPROM from 0x0C
 * 				  Should increment value of next location.
 */
defbyte ()
{

						if (TRACE){
													  fprintf(output,"\t;defbyte()\n");
													  }

										  fprintf(output,"\tequ\th'0x%X'",avail_store_addr);
										  avail_store_addr++;
}

/*
 * 				  print pseudo-op to define storage
 * 				  NOTE change this routine to take the value.
 */
defstorage ()
{
						if (TRACE)
										  {			fprintf(output,"\t;defstorage()\n");
														fprintf(output,"\t;avail_store_addr = $%X",avail_store_addr);
										  }

								  fprintf(output,"\tequ\th'0x%X'",avail_store_addr);
								  avail_store_addr++;
} // end of defstorage

/*
 * 				  print pseudo-op to define a word
 *
 */
defword ()
{
						if (TRACE)
													  fprintf(output,"\t;defword()\n");
						ot ("equ\t");
}

/*
 * 				  modify the stack pointer to the new value indicated
 *
 */
modstk (int newstkp)
{
						int			  k;
																char buffer[50];

						if (TRACE)
													  fprintf(output,"\t;modstk(%d)\n",newstkp);

						k = galign(newstkp - stkp);

						if (k == 0) {
													  return (newstkp);
						}
						/* forget about stack alignment */
						if (k > 0) {
																					  sprintf(buffer,"addf stackptr, #%d",k);
																					  ol(buffer);
								 return (newstkp);
						}
						else {
																					  k=-k;
																					  sprintf(buffer,"sub stackptr, #%d",k);
																					  ol(buffer);
								 return (newstkp);
						}
}

/*
 * 				  multiply the primary register by INTSIZE
 */
gaslint ()
{
						if (TRACE)
													  fprintf(output,"\t;gaslint()\n");
						/* ol ("dad\th"); */
}

/*
 * 				  divide the primary register by INTSIZE
 */
gasrint()
{
						if (TRACE)
								 fprintf(output,"\t;gasrint()\n");
						return;
#ifdef INTEGERS
						gpush(); 															/* push primary in prep for gasr */

						if (TRACE)
								{
								fprintfoutput,"\t immed called gasrint()");
								}
						immed ();
						onum (1);
						nl ();
						gasr (); 	  /* divide by two */
#endif
}

/*
 * 				  Case jump instruction
 */

gjcase(int label)
{
						if (TRACE)
													  fprintf(output,"\t;gjcase()\n");
						jump(label);
}

/* this is a real hack */
/* output inline code for case statement */

outcase(int typ,
						int value,
						int label)
{
								char buffer[50];

								if (TRACE)
														fprintf(output,"\t;outcase()\n");
								if (typ==CASE) {

													  sprintf(buffer,"movlw %d",typ);
													  sprintf(buffer,"subwf %d,0",value);
													  sprintf(buffer,"btfsc 3,2");
													  sprintf(buffer,"goto %d",label);
													  //sprintf(buffer,"cje primary,#%d, _%d_",value,label);
													  //ol(buffer);
								}
								else
								if (typ==DEFAULT) {
													  sprintf(buffer,"goto %d",label);
													  ol(buffer);
								}
								else
													  outstr(ERRMSG);
}

/*
 * 				  add the primary and secondary registers
 * 				  if lval2 is int pointer and lval is not, scale lval
								 A right pain as there is only one register!
 */

//gadd (int lval, int *lval2)
gadd (int lval)
{
						if (TRACE)
								 {
								// fprintf(output,"\t;gadd(%d, %d)\n",&lval,lval2);
								 fprintf(output,"\t;gadd num %d $%x ptr = %p &ptr = %p \n",lval,lval,lval,&lval);
								 printf("gadd num %d $%x ptr = %p &ptr = %p \n",lval,lval,lval,&lval);
								 }
						//gpop ();
						//if (dbltest (lval2, lval)) {
						// 						 swap ();
						// 						 gaslint ();
						// 						 swap ();
						// }

						fprintf(output,"\taddwf %d\n",lval);
}

/*
 * 				  subtract the primary register from the secondary
 * 				  P=S-P;
 * 				  Assumes W and second are loaded on call.
 */
gsub ()
{
						if (TRACE)

						fprintf(output,"\t;gsub()\n");

						ol("subwf second");	// easy enough?

						//gpop ();
						// 							S_lib=TRUE;
						// 							gcall("sub");
/*****
						ol ("movf temp, secondary");
						ol ("sub temp, primary");
						ol ("movf primary, temp");
******/
}

/*
 * 				  multiply the primary and secondary registers
 * 				  (result in primary)
 *
 */
gmult ()
{
						if (TRACE)
													  fprintf(output,"\t;gmult()\n");
																M_lib=TRUE;
						gpop();
						gcall ("_mul");
}

/*
 * 				  divide the secondary register by the primary
 * 				  (quotient in primary, remainder in secondary)
 *
 */
gdiv ()
{
						if (TRACE)
													  fprintf(output,"\t;gdiv()\n");
																M_lib=TRUE;
						gpop();
						gcall ("_div");
}

/*
 * 				  compute the remainder (mod) of the secondary register
 * 				  divided by the primary register
 * 				  (remainder in primary, quotient in secondary)
 *
 */

gmod ()
{
						if (TRACE)
													  fprintf(output,"\t;gmod()\n");
																M_lib=TRUE;
						gdiv ();
						swap ();
}

/*
 * 				  inclusive 'or' the primary and secondary registers
 *
 */
gor ()
{
						if (TRACE)
													  fprintf(output,"\t;gor()\n");
						gpop();
						ol ("iorwf second,W ");
}

/*
 * 				  exclusive 'or' the primary and secondary registers
 *
 */
gxor ()
{
						if (TRACE)
								 fprintf(output,"\t;gxor()\n");
						gpop();
						ol ("xorwf second,W");
}

/*
 * 				  'and' the primary and secondary registers
 *
 */
gand ()
{
						if (TRACE)
													  fprintf(output,"\t;gand()\n");
						gpop();
						ol ("andwf second,W");
}

/*
 * 				  arithmetic shift right the secondary register the number of
 * 				  times in the primary register
 * 				  (results in primary register)
 *
 */
gasr ()
{
						if (TRACE)
													  fprintf(output,"\t;gasr()\n");
						ol("rep:   rrf second");
						ol("decfsz W");
						ol("goto rep");
						ol("movlf second,w");
						ol("return");

}

/*
 * 				  arithmetic shift left the secondary register the number of
 * 				  times in the primary register
 * 				  (results in primary register)
 *
 */
gasl ()
{
						if (TRACE)
								 fprintf(output,"\t;gasl()\n");
						ol("rep:   rlf second");
						ol("decfsz W");
						ol("goto rep");
						ol("movlf second,w");
						ol("return");
}

/*
 * 				  two's complement of primary register
 * 				  IN PIC this is comf w,0 (complement W)
 * 				  Then add 1.
 */
gneg ()
{
						if (TRACE)
													  fprintf(output,"\t;gneg()\n");
						ol ("comf W,0 ");
						ol ("addlw 1");

}

/*
 * 				  logical complement of primary register
 * 				  Easy enough with the PIC!
 * 				  COMF does the trick!
 */
glneg ()
{
						if (TRACE)
								 fprintf(output,"\t;glneg()\n");
						ol ( "comf w,w");
}

/*
 * 				  one's complement of primary register
 *
 */
gcom ()
{
						if (TRACE)
								 fprintf(output,"\t;gcom()\n");
						ol ("comf w,w");
}

/*
 * 				  Convert primary value into logical value (0 if 0, 1 otherwise)
 * 				  Think this will work. Needs checking.
 */
gbool (int *lval)
{
						if (TRACE)
													  fprintf(output,"\t;gbool()\n");
						fprintf(output,"\tmovlw %d",lval);
						ol("btfsc 3,2");
						ol("retlw 0"); 		 // 0 in W
						ol("btfss 3,2");
						ol("retlw 1"); 		 // 1 in W
}

/*
 * 				  increment the primary register by 1 if char, INTSIZE if
 * 				  int
 */
ginc (int lval[])
{
						if (TRACE)
													  fprintf(output,"\t;ginc()\n");
						ol ("incf W,W");
						if (lval[2] == CINT) {
outstr( ERRMSG );
								 ol ("addlw 2");
						}

}

/*
 * 				  decrement the primary register by one if char, INTSIZE if
 * 				  int
 */
gdec (int lval[])
{
						if (TRACE) fprintf(output,"\t;gdec()\n");
						ol ("decf W,W");
						if (lval[2] == CINT) {
outstr( ERRMSG );
								 ol("sublw 2");
						}
}

/*
 * 				  following are the conditional operators.
 * 				  they compare the secondary register against the primary register
 * 				  and put a literl 1 in the primary if the condition is true,
 * 				  otherwise they clear the primary register
 *
 */

/*
 * 				  equal
 *
 */
geq ()
{
						if (TRACE)
													  fprintf(output,"\t;geq()\n");
						ol("subwf second W");
						ol("btfsc 3,2");
						ol("retlw 0"); 		 // W is 0
						ol("retlw 1");
}

/*
 * 				  not equal
 *
 */
gne ()
{
						if (TRACE)
								 fprintf(output,"\t;gne()\n");
						ol("subwf second,W");
						ol("btfss 3,2");
						ol("retlw 1");
						ol("retlw 0");
}

/* ******  signed chars are not handled yet, use unsigned versions ******/
/*
 * 				  less than (signed)
 *
 */

glt ()
{
						if (TRACE)
													  fprintf(output,"\t;glt()\n");
outstr( WARNING );
						gpop();
																G_lib=TRUE;
																gcall( "_glt");
/********
						ol("cjb secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
********/
}


/*
 * 				  less than or equal (signed)
 *
 */
gle ()
{
						if (TRACE)
													  fprintf(output,"\t;gle()\n");
outstr( WARNING );
						gpop();
																G_lib=TRUE;
																gcall("_gle");
/********
						ol("cjbe secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
*********/
}

/*
 * 				  greater than (signed)
 *
 */
ggt ()
{
						if (TRACE)
													  fprintf(output,"\t;ggt()\n");
outstr( WARNING );
						gpop();
																G_lib=TRUE;
																gcall("_ggt");
/********
						ol("cja secondary, primary, $+7");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
********/
}

/*
 * 				  greater than or equal (signed)
 *
 */
gge ()
{
						if (TRACE)
													  fprintf(output,"\t;gge()\n");
outstr( WARNING );
						gpop();
																G_lib=TRUE;
																gcall ("_gge");

/******
						ol("cjae secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
*******/
}

/*
 * 				  less than (unsigned)
 *
 */
gult ()
{
						if (TRACE)
													  fprintf(output,"\t;gult()\n");
						gpop();
																G_lib=TRUE;
						gcall ("_gult");
/*******
						ol("cjb secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
*******/
}

/*
 * 				  less than or equal (unsigned)
 *
 */
gule ()
{
						if (TRACE)
								 fprintf(output,"\t;gule()\n");
						gpop();
																G_lib=TRUE;
						gcall ("_gule");
/*******
						ol("cjbe secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
*******/
}

/*
 * 				  greater than (unsigned)
 *
 */
gugt ()
{
						if (TRACE)
													  fprintf(output,"\t;gugt()\n");
						gpop();
																G_lib=TRUE;
						gcall ("_gugt");
/********
						ol("cja secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
*********/

}

/*
 * 				  greater than or equal (unsigned)
 *
 */
guge ()
{
						if (TRACE)
													  fprintf(output,"\t;guge()\n");
						gpop();
																G_lib=TRUE;
						gcall ("_guge");
/**********
						ol("cjae secondary, primary, $+7 ");
						ol("movf primary, #0");
						ol("skip");
						ol("movf primary, #1");
**********/

}

inclib() {
						return(INCDIR);
}


/* 				  Squirrel away argument count in a register that modstk
						doesn't touch.
*/

gnargs(int d)
{
						if (TRACE)
													  fprintf(output,"\t;gnargs(%d)\n",d);
						ot (";;(# args passed) movf, #");
						onum(d);
						nl ();
}

assemble(char *s)
{
						if (TRACE)
													  fprintf(output,"\t;assemble(%s)\n",s);
#ifdef  ASNM
						char buf[100];
						strcpy(buf, ASNM);
						strcat(buf, " ");
						strcat(buf, asm);
						buf[strlen(buf)-1] = 's';
						return(system(buf));
#else
						return(0);
#endif
}

link() {
#ifdef  LDNM
						fputs("I don't know how to link files yet\n", stderr);
#else
						return(0);
#endif
}

